Attribute VB_Name = "modIFileDialogEvents"
' Autor: F. Schler (frank@activevb.de)
' Datum: 02/2021

Option Explicit

' ----==== Const ====----
Private Const S_OK As Long = &H0
Private Const E_NOINTERFACE As Long = &H80004002
Private Const IID_IFileDialogEvents As String = "{973510DB-7D7F-452B-8975-74A85828D354}"

' ----==== Types ====----
Private Type tIFileDialogEvents
    pVTable As Long
End Type

Private Type tIFileDialogEventsVTable
    VTable(0 To 9) As Long
End Type

Private Type GUID
    data1 As Long
    data2 As Integer
    data3 As Integer
    data4(0 To 7) As Byte
End Type

' ----==== Kernel32 API-Deklarationen ====----
Private Declare Sub CopyMemory Lib "Kernel32.dll" _
                    Alias "RtlMoveMemory" ( _
                    ByRef Destination As Any, _
                    ByRef Source As Any, _
                    ByVal length As Long)

Private Declare Function lstrlenW Lib "Kernel32.dll" ( _
                         ByVal lpString As Long) As Long

' ----==== Ole32 API-Deklarationen ====----
Private Declare Sub CoTaskMemFree Lib "Ole32.dll" ( _
                    ByVal hMem As Long)

Private Declare Function StringFromGUID Lib "Ole32.dll" _
                         Alias "StringFromIID" ( _
                         ByRef pGUID As GUID, _
                         ByRef lpszProgID As Long) As Long

' ----==== Variablen ====----
Private m_IPtr As Long
Private m_RefCount As Long
Private m_Interface As Object
Private m_IFileDialogEvents As tIFileDialogEvents
Private m_IFileDialogEventsVTable As tIFileDialogEventsVTable

Public Function IFileDialogEvents(ByVal cIFileDialogEvents As Long) As Long

    ' Objekt vom Pointer
    Call CopyMemory(m_Interface, cIFileDialogEvents, 4)

    ' ----==== Interface IFileDialogEvents  ====----
    With m_IFileDialogEventsVTable
        
        ' VTable aufbauen
        .VTable(0) = ProcPtr(AddressOf QueryInterface)
        .VTable(1) = ProcPtr(AddressOf AddRef)
        .VTable(2) = ProcPtr(AddressOf Release)
        .VTable(3) = ProcPtr(AddressOf OnFileOk)
        .VTable(4) = ProcPtr(AddressOf OnFolderChanging)
        .VTable(5) = ProcPtr(AddressOf OnFolderChange)
        .VTable(6) = ProcPtr(AddressOf OnSelectionChange)
        .VTable(7) = ProcPtr(AddressOf OnShareViolation)
        .VTable(8) = ProcPtr(AddressOf OnTypeChange)
        .VTable(9) = ProcPtr(AddressOf OnOverwrite)

    End With

    With m_IFileDialogEvents

        ' Pointer auf die VTable
        .pVTable = VarPtr(m_IFileDialogEventsVTable)

    End With

    ' Pointer auf das Interface
    m_IPtr = VarPtr(m_IFileDialogEvents)

    IFileDialogEvents = m_IPtr

End Function

' ----==== Hilfsfunktionen ====----
Private Function ProcPtr(ByVal ptr As Long) As Long

    ProcPtr = ptr

End Function

Private Function Guid2Str(ByRef tGUID As GUID) As String

    Dim lGuid As Long

    If StringFromGUID(tGUID, lGuid) = S_OK Then

        Guid2Str = StrPtr2Str(lGuid)
    
    End If

End Function

Private Function StrPtr2Str(ByVal lpStrPointer As Long) As String

    Dim lLen As Long
    Dim bBuffer() As Byte

    If lpStrPointer = 0 Then Exit Function

    lLen = lstrlenW(lpStrPointer) * 2& - 1&

    If lLen > 0& Then

        ReDim bBuffer(lLen)

        Call CopyMemory(bBuffer(0), ByVal lpStrPointer, lLen)
        
        Call CoTaskMemFree(lpStrPointer)

        StrPtr2Str = bBuffer

    End If

End Function

' ----==== IUnknown Funktionen ====----
Private Function QueryInterface(ByVal this As Long, ByRef riid As GUID, ByRef _
    pvObj As Long) As Long

    Dim lRet As Long
    
    Select Case UCase$(Guid2Str(riid))

    ' wenn nach dem Interface IFileDialogEvents gefragt wird
    Case UCase$(IID_IFileDialogEvents)

        ' dann muss ein AddRef aufgerufen werden.
        Call AddRef(this)
        
        ' Pointer auf das Interface
        pvObj = m_IPtr
        
        ' OK zurck geben, Interface ist vorhanden
        lRet = S_OK

    Case Else
        
        ' alles andere ignorieren
        
        pvObj = 0&
        
        lRet = E_NOINTERFACE
    
    End Select

    QueryInterface = lRet

End Function

Private Function AddRef(ByVal this As Long) As Long

    m_RefCount = m_RefCount + 1
    
    AddRef = m_RefCount

End Function

Private Function Release(ByVal this As Long) As Long
    
    m_RefCount = m_RefCount - 1
    
    If m_RefCount = 0 Then
    
        ' alles Aufrumen und Freigeben
        m_IFileDialogEvents.pVTable = 0&
        Erase m_IFileDialogEventsVTable.VTable
        
        m_IPtr = 0&
        
        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)
        
    End If
    
    Release = m_RefCount

End Function

' ----==== Interface IFileDialogEvents Func ====----
Private Function OnFileOk(ByVal this As Long, ByVal pfd As Long) As Long

    ' Funktion in der Klasse aufrufen
    Call m_Interface.OnFileOk
    
End Function

Private Function OnFolderChanging(ByVal this As Long, ByVal pfd As Long, ByVal _
    psiFolder As Long) As Long
    
    ' Funktion in der Klasse aufrufen
    Call m_Interface.OnFolderChanging(psiFolder)
    
End Function

Private Function OnFolderChange(ByVal this As Long, ByVal pfd As Long) As Long

    ' Funktion in der Klasse aufrufen
    Call m_Interface.OnFolderChange
    
End Function

Private Function OnSelectionChange(ByVal this As Long, ByVal pfd As Long) As Long

    ' Funktion in der Klasse aufrufen
    Call m_Interface.OnSelectionChange
    
End Function

Private Function OnShareViolation(ByVal this As Long, ByVal pfd As Long, ByVal psi _
    As Long, ByVal pResponse As Long) As Long
    
    ' Funktion in der Klasse aufrufen
    Call m_Interface.OnShareViolation(psi, pResponse)
    
End Function

Private Function OnTypeChange(ByVal this As Long, ByVal pfd As Long) As Long

    ' Funktion in der Klasse aufrufen
    Call m_Interface.OnTypeChange
    
End Function

Private Function OnOverwrite(ByVal this As Long, ByVal pfd As Long, ByVal psi As _
    Long, ByVal pResponse As Long) As Long
    
    ' Funktion in der Klasse aufrufen
    Call m_Interface.OnOverwrite(psi, pResponse)
    
End Function

